/**
 * Copyright (c) 2008-2010 The Open Source Geospatial Foundation
 * 
 * Published under the BSD license.
 * See http://svn.geoext.org/core/trunk/geoext/license.txt for the full text
 * of the license.
 */

Ext.namespace("GeoExt.ux")

/*
 * @requires GeoExt/widgets/Action.js
 */

/** api: (define)
 *  module = GeoExt.ux
 *  class = ZoomTo
 */

/** api: constructor
 *  .. class:: ZoomTo 
 */
GeoExt.ux.ZoomTo = Ext.extend(Ext.Action, {

    /* begin i18n */
    /** api: config[zoomToText] ``String`` i18n */
    zoomToText: "ZoomTo",

    /** api: config[xCoordinateText] ``String`` i18n */
    xCoordinateText: "X coordinate",

    /** api: config[yCoordinateText] ``String`` i18n */
    yCoordinateText: "Y coordinate",

    /** api: config[projectionText] ``String`` i18n */
    projectionText: "Projection",

    /** api: config[invalidEntryText] ``String`` i18n */
    invalidEntryText: "invalid entry",

    /** api: config[widgetTitleText] ``String`` i18n */
    widgetTitleText: "ZoomTo widget",

    /** api: config[destroyMarkerActionText] ``String`` i18n */
    destroyMarkerActionText: "Remove +",

    /** api: config[closeActionText] ``String`` i18n */
    closeActionText: "Close",

    /** api: config[zoomActionText] ``String`` i18n */
    zoomActionText: "Zoom",

    /** api: config[errorText] ``String`` i18n */
    errorText: "Error",

    /** api: config[missingProjectionText] ``String`` i18n */
    missingProjectionText: "The projection is missing.",

    /** api: config[missingCoordsText] ``String`` i18n */
    missingCoordsText: "Missing or invalid coordinates.",

    /** api: config[outOfRangCoordsText] ``String`` i18n */
    outOfRangCoordsText: "Coordinates are out of current extent range.",
    /* end i18n */

    /** private: property[window]
     *  ``Ext.Window``
     */
    window: null,

    /** private: property[form]
     *  ``Ext.form.FormPanel``
     */
    form: null,

    /** private: property[map]
     *  ``OpenLayers.Map``
     */
    map: null,

    /** private: property[projectionStore]
     *  ``Ext.data.SimpleStore``
     */
    projectionStore: null,

    /** private: property[defaultZoomLevel]
     *  ``Integer``
     */
    defaultZoomLevel: null,

    /** private: property[autoHideWindowOnZoom]
     *  ``Boolean``
     *  If set to true, hides the window after recentering.
     */
    autoHideWindowOnZoom: true,

    /** private: property[useIcons]
     *  ``Boolean``
     *  If set to true, enables the use of image icons.  Must be combined with
     *  a .css (see in resources/css).
     */
    useIcons: true,

    /** private: property[showCenter]
     *  ``Boolean``
     *  If set to true, shows a red cross marker on the center of the map
     *  after recentering.
     */
    showCenter: true,

    /** private: method[constructor]
     */
    constructor: function(config) {
        Ext.apply(this, config);

        this.initMap();
        arguments.callee.superclass.constructor.call(this, config);
        this.setHandler(this.onActionClick, this);

        if (this.useIcons === true) {
            this.setIconClass("gx-zoomto-recenteroncoords");
        } else {
            this.setText(this.zoomToText);
        }
    },

    /** private: method[initMap]
     *  Convenience method to make sure that the map object is correctly set.
     */
    initMap: function() {
        if (this.map instanceof GeoExt.MapPanel) {
            this.map = this.map.map;
        }

        if (!this.map) {
            this.map = GeoExt.MapPanel.guess().map;
        }

        // if no toggleGroup was defined, set to this.map.id
        if (!this.toggleGroup) {
            this.toggleGroup = this.map.id;
        }
    },

    /** private: method[onActionClick]
     *  Called when action is clicked.
     */
    onActionClick: function() {
        if (!this.window) {

            if (!this.projectionStore) {
                this.projectionStore = new Ext.data.SimpleStore({
                    fields: ['projection'],
                    data : [
                        ['EPSG:4326'],
                        ['EPSG:900913']
                    ]
                });
            }
                        
            // get first value of store as default value
            var defaultProj = this.projectionStore.data.items[0].data[
                this.projectionStore.getAt(0).fields.get(0)['name']
            ];

            // FormPanel
            this.form = new Ext.form.FormPanel({
                border: false,
                plain: true,
                defaults: {width: 170},
                defaultType: 'textfield',
                labelAlign: 'top',
                bodyStyle:'padding:5px 5px 0',
                items: [{
                        fieldLabel: this.xCoordinateText,
                        name: 'coordx'
                    },{
                        fieldLabel: this.yCoordinateText,
                        name: 'coordy' 
                    },{
                        fieldLabel: this.projectionText,
                        name: 'projection',
                        value: defaultProj,
                        xtype: 'combo',
                        store: this.projectionStore,
                        displayField: 'projection',
                        typeAhead: true,
                        mode: 'local',
                        forceSelection: true,
                        triggerAction: 'all',
                        allowBlank: false,
                        invalidText: this.invalidEntryText,
                        emptyText: "",
                        selectOnFocus:true
                    }
                ]
            });

            var windowButtons = [];

            // remove marker button (optional)
            if (this.showCenter === true) {
                windowButtons.push({
                    text: this.destroyMarkerActionText,
                    scope: this,
                    handler: function(){
                        this.destroyMarker();
                    }
                });
            }

            // close button
            /*
            windowButtons.push({
                text: this.closeActionText,
                scope: this,
                handler: function(){
                    this.window.hide();
                }
            });
            */

            // zoom button
            windowButtons.push({
                text: this.zoomActionText,
                scope: this,
                handler: function(){
                    this.recenter();
                }
            });

            // Window with buttons
            this.window = new Ext.Window({
                title: this.widgetTitleText,
                layout: 'form',
                width: 200,
                autoHeight: true,
                height: 'auto',
                closeAction: 'hide',
                modal: false,
                plain: false,
                resizable: false,
                buttonAlign: 'center',
                items: [this.form],
                buttons: windowButtons
            });
        }

        this.window.show();
    },

    /** private: method[recenter]
     * Recenters map using user-provided coordinates and scale.
     */
    recenter: function() {
        var values = this.form.getForm().getValues();

        if (values.projection == "") {
            this.showError(this.missingProjectionText);
            return false;
        }

        var transformedCoords = this.getTransformedCoords(
            values.coordx, values.coordy, values.projection
        );

        var x = transformedCoords['x'];
        var y = transformedCoords['y'];
        
        if (this.checkCoords(x, y)) {
            
            var zoom;

            if (this.scales && values.scaleValue) {
                // use user-provided scale
                resolution = OpenLayers.Util.getResolutionFromScale(values.scaleValue,
                this.map.units);
                zoom = this.map.getZoomForResolution(resolution);
            } else if (this.defaultZoomLevel) {
                zoom = this.defaultZoomLevel;
            }

            this.recenterOnCoords(x, y, zoom);
        }
    },

    /** private: method[checkCoords]
     *  Checks that submitted coordinates are well-formatted and within the map
     *      bounds.
     *
     *  Parameters:
     *  x    {Float}  - easting coordinate
     *  y    {Float}  - northing coordinate
     *  proj {String} - projection of inputed coordinates
     *
     *  Returns: 
     *  {Object}
     */
    getTransformedCoords: function(x, y, proj) {
        var transformedCoords = {};

        if (this.map.getProjection() === proj) {
            transformedCoords['x'] = x;
            transformedCoords['y'] = y;
        } else {
            var objProj = new OpenLayers.Projection(proj);
            var point = new OpenLayers.LonLat(x, y);
            point.transform(
                objProj,
                new OpenLayers.Projection(this.map.getProjection())
            );

            transformedCoords['x'] = point.lon;
            transformedCoords['y'] = point.lat;
        }

        return transformedCoords;
    },

    /** private: method[checkCoords]
     *  Checks that submitted coordinates are well-formatted and within the map
     *      bounds.
     *
     *  Parameters:
     *  x {Float} - easting coordinate
     *  y {Float} - northing coordinate
     *
     *  Returns: 
     *  {Boolean}
     */
    checkCoords: function(x, y) {
    
        if (!x || !y) {
            this.showError(this.missingCoordsText);
            return false;
        }

        var maxExtent = this.map.getMaxExtent();
    
        if (x < maxExtent.left || x > maxExtent.right ||
            y < maxExtent.bottom || y > maxExtent.top) {
            this.showError(this.outOfRangCoordsText);
            return false;
        }
    
        return true;
    },

    /** private: method[recenterOnCoords]
     *  Recenters on given coordinates and zoom level
     *
     *  Parameters:
     *  x -    {Float}   easting coordinate
     *  y -    {Float}   northing coordinate
     *  zoom - {Integer} zoom level (optional)
     */
    recenterOnCoords: function(x, y, zoom) {
        
        // use default zoom level if provided in widget config, 
        // else keep current zoom level
        if (typeof(zoom) == 'undefined') {
            zoom = (typeof(this.defaultZoom) != 'undefined') 
                   ? this.defaultZoom : this.map.getZoom()
        }

        if (this.showCenter) {
            // display a symbol on the new center point
            this.showCenterMark(x, y);
        }

        this.map.setCenter(new OpenLayers.LonLat(x, y), zoom);

        if (this.autoHideWindowOnZoom === true) 
        {
            this.window.hide();
        }
    },

    /** private: method[showCenterMark]
     *  Materializes new center with a cross
     *
     *  Parameters:
     *  x - {Float} easting coordinate
     *  y - {Float} northing coordinate
     */
    showCenterMark: function(x, y) {
        this.prepareVectorLayer();

        var features = [
            new OpenLayers.Feature.Vector(
                new OpenLayers.Geometry.Point(x, y),
                { type: this.symbol || 'cross' }
            )
        ];

        this.vectorLayer.addFeatures(features);
    },

    /** private: method[prepareVectorLayer]
     *  Adds a layer for displaying the center symbol. If it is already set,
     *  removes existing features.
     */
    prepareVectorLayer: function() {
        if (this.vectorLayer) {
            this.destroyMarker();
        } else {
            var styles = new OpenLayers.StyleMap({
                "default": OpenLayers.Util.extend({
                    // retrieved from symbol type attribute
                    graphicName: "${type}",
                    pointRadius: 10,
                    fillColor: "red",
                    fillOpacity: 1
                }, this.centerMarkStyles)
            });

            this.vectorLayer = new OpenLayers.Layer.Vector(
                "Center Symbol", {
                    styleMap: styles,
                    alwaysInRange: true
                }
            );

            this.map.addLayer(this.vectorLayer);
        }
    },

    /** private: method[destroyMarker]
     *  Removes existing features from vector layer
     */
    destroyMarker: function() {
        if (this.vectorLayer) {
            this.vectorLayer.destroyFeatures();
        }
    },

    /** private: method[showError]
     *  Displays an error message
     *
     *  Parameters:
     *  msg - {String} message
     *  title - {String} box title
     */
    showError: function(msg, title) {
        title = title || this.errorText;
        Ext.Msg.alert(title, msg);
    }
});
