<?php
/**
* Authentification par josso
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-authentication
* @author    Pascal Martin 
* @copyright Boréal Informations Stratégiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/

/**
* Authentification par josso  
* 
* @category   PHP 
* @package    Borealis-authentication
* @subpackage Checker  
* @author     Pascal Martin 
*/ 
class com_borealis_authentication_checker_JossoChecker extends com_borealis_authentication_checker_Checker
{
    private $m_strUrl = null;
    private $m_strSecurityDomain = null;
    private $m_objSoapClientIdentityProvider = null;
    
    /**
    * Constructeur de la classe com_borealis_authentication_checker_JossoChecker 
    * 
    * @param com_borealis_authentication_password_PasswordCreator $pobjPasswordCreator Créateur de mot de passe 
    * @param string                                               $pstrUrl             Url du service web josso d'authetification
    * @param string                                               $pstrSecurityDomain  Security Domain
    * 
    * @return com_borealis_authentication_checker_JossoChecker
    */
    public function __construct(com_borealis_authentication_password_PasswordCreator $pobjPasswordCreator, $pstrUrl, $pstrSecurityDomain) 
    {        
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrUrl);
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrSecurityDomain);
        
        $this->m_strUrl = $pstrUrl;
        $this->m_strSecurityDomain = $pstrSecurityDomain;
        parent::__construct($pobjPasswordCreator);
    }
    
    /**
    * Vérifie le doit d'authentification
    * 
    * @param string $pstrUsername Nom d'utilisateur
    * @param string $pstrPassword Mot de passe
    * 
    * @return boolean Authorized 
    */
    public function isAuthorized($pstrUsername, $pstrPassword)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrUsername);
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrPassword);
        
        try
        {
            $strAssertionId = $this->assertIdentityWithSimpleAuthentication($pstrUsername, $this->getPasswordValue($pstrPassword));
            $strToken = $this->resolveAuthenticationAssertion($strAssertionId); 
            if (isset($_SESSION))
            { 
                $_SESSION['JOSSO_TOKEN'] = $strToken;            
            }
            return true;
        }
        catch(SoapFault $objError)
        { 
            if (isset($_SESSION))
            {
                $_SESSION['JOSSO_TOKEN_REASON'] = (string)$objError;         
                unset($_SESSION['JOSSO_TOKEN']);                                       
            }
            return false;
        }
    }    
    
    /**
    * Call web method assertIdentityWithSimpleAuthentication and return assertionId
    *     
    * @param string $pstrUsername Nom d'utilisateur
    * @param string $pstrPassword Mod de passe
    * 
    * @return string assertionId
    */
    private function assertIdentityWithSimpleAuthentication($pstrUsername, $pstrPassword)
    {    
        $objSoapClient = $this->getSoapClientIdentityProvider();
            
        $objSoapVarSecurityDomain = new SoapVar($this->m_strSecurityDomain, XSD_STRING, null, null, 'securityDomain');
        $objSoapVarUsername = new SoapVar($pstrUsername, XSD_STRING, null, null, 'username');
        $objSoapVarPassword = new SoapVar($pstrPassword, XSD_STRING, null, null, 'password');
        
        $objSoapVarRequest = new SoapVar(array($objSoapVarSecurityDomain, $objSoapVarUsername, $objSoapVarPassword), SOAP_ENC_OBJECT, 'AssertIdentityWithSimpleAuthenticationRequestType', 'http://josso.org/gateway/ws/1.1/protocol', 'AssertIdentityWithSimpleAuthenticationRequest');
        $objArrayParams = array($objSoapVarRequest);
       
        $objResult = $this->soapCall($objSoapClient, 'assertIdentityWithSimpleAuthentication', $objArrayParams);  
        return $objResult->assertionId;              
    }
    
    /**
    * Call web method resolveAuthenticationAssertion and return ssoSessionId    
    * 
    * @param string $pstrAssertionId assertionId
    * 
    * @return string ssoSessionId
    */
    private function resolveAuthenticationAssertion($pstrAssertionId)
    {        
        $objSoapClient = $this->getSoapClientIdentityProvider();
            
        $objSoapVarAssertionId = new SoapVar($pstrAssertionId, XSD_STRING, null, null, 'assertionId');
        $objSoapVarRequest = new SoapVar(array($objSoapVarAssertionId), SOAP_ENC_OBJECT, 'ResolveAuthenticationAssertionRequestType', 'http://josso.org/gateway/ws/1.1/protocol', 'ResolveAuthenticationAssertionRequest');
        $objArrayParams = array($objSoapVarRequest);
                 
        $objResult = $this->soapCall($objSoapClient, 'resolveAuthenticationAssertion', $objArrayParams);  
        return $objResult->ssoSessionId;                  
    }
    
    /**
    * Return Soap Client with IdentityProvider location
    * 
    * @return SoapClient
    */    
    private function getSoapClientIdentityProvider()
    {        
        if ($this->m_objSoapClientIdentityProvider === null)
        {
            $objArrayOptions = array(
                                'encoding'=>'ISO-8859-1',
                                'connection_timeout'=> 0,
                                'exceptions' => true,            
                                'uri'  => 'http://josso.org/gateway/ws/1.1/wsdl/soapbinding/IdentityProvider', 
                                'location' => $this->m_strUrl);                                 
            $this->m_objSoapClientIdentityProvider = new SoapClient(null, $objArrayOptions); 
        }     
        return $this->m_objSoapClientIdentityProvider;        
    } 
    
    /**
    * Call web method
    * 
    * @param SoapClient $pobjSoapClient  Soap client
    * @param string     $pstrMethod      Method name
    * @param array      $pobjArrayParams Method param
    * 
    * @return void
    */
    private function soapCall(SoapClient $pobjSoapClient, $pstrMethod, $pobjArrayParams)
    {
        try
        {
            ini_set('default_socket_timeout', 2);
            $objResult = $pobjSoapClient->__soapCall($pstrMethod, $pobjArrayParams);            
            ini_restore('default_socket_timeout'); 
            return $objResult; 
        }
        catch(SoapFault $objError)
        {     
            ini_restore('default_socket_timeout');
            throw $objError;
        }         
    } 
}
  
?>
