/**
 * Copyright (c) 2009-2012 Mapgears Inc.
 * 
 * Published under the BSD license.
 * See http://geoprisma.org/license for the full text of the license. 
 */

Ext.namespace("org.GeoPrisma");

/** api: (define)
 *  module = org.GeoPrisma
 *  class = API
 */

/** api: constructor
 *  .. class:: API
 */
org.GeoPrisma.API = Ext.extend(Ext.util.Observable, {

    // API Properties

    /** api: property[version]
     * :class:``GeoExt.MapPanel``
     * A reference to the MapPanel widget.  If none is providen, then it's
     * guessed.
     */
    mapPanel: null,

    /** api: property[version]
     * ``String``
     * The version number of GeoPrisma.
     */
    version: null,

    // Private

    /** private: method[constructor]
     */
    constructor: function(config) {
        Ext.apply(this, config);
        arguments.callee.superclass.constructor.call(this, config);

        this.addEvents(
            /** private: event[applyfilters]
             *  Fires when applying one or more filters to one or more
             *  resources.
             */
            "applyfilters"
        );
    },

    // Public Methods (API)

    /** public: method[applyFilters]
     *  :param resourceFilters: ``Array``
     *    An array of hash objects that containing the following keys :
     *    - 'resource' : the name of the resource to apply the filter
     *    - 'filter'   : the filter to apply to the resource
     *
     *  This method fires the 'applyfilters' event with the array as 
     *  argument.
     */
    applyFilters: function(resourceFilters) {
        this.fireEvent("applyfilters", resourceFilters);
    },

    /** public: method[applyFilter]
     *  :param resourceFilter: ``Object``
     *    Hash object that contains the following keys :
     *    - 'resource' : the name of the resource to apply the filter
     *    - 'filter'   : the filter to apply to the resource
     *
     *  This method fires the 'applyfilters' event with the resourceFilter added
     *  to an array as the argument.
     */
    applyFilter: function(resourceFilter) {
        this.fireEvent("applyfilters", [resourceFilter]);
    },

    // Public Methods (Util)

    /** public: method[getLayers]
     *  :param option: ``Object``
     *    Hash object containing search options.  Possible keys (all optional)
     *    are :
     *    - resourceName (String) : get layers having the specific resource name
     *    - serviceType (String): get layers only of specific service type
     *  :return:  ``Array`` of :class:``OpenLayers.Layer`` objects.
     *
     *  Returns layers that match the search options defined.
     */
    getLayers: function(options) {
        options = options || {};

        // make sure the mapPanel property is set
        if (!this.mapPanel) {
            this.setMapPanel();
        }

        var layers = this.mapPanel.map.layers;

        // search each criteria defined
        if (options.resourceName) {
            layers = this.getLayersByResourceName(options.resourceName, layers);
        }
        if (options.serviceType) {
            layers = this.getLayersByServiceType(options.serviceType, layers);
        }

        return layers;
    },

    /** public: method[getLayersByResourceName]
     *  :param match: ``String`` resource name the layer must match
     *  :param layers: ``Array`` of :class:``OpenLayers.Layer`` objects.  If not
     *                           defined, then use all map layers.
     *  :return:  ``Array`` of :class:``OpenLayers.Layer`` objects.
     *
     *  Returns layers that matching the specified resource name.
     */
    getLayersByResourceName: function(match, layers) {
        layers = layers || this.mapPanel.map.layers;
        var found = [];
        Ext.each(layers, function(l) {
            l.resources && l.resources.indexOf(match) != -1 && found.push(l);
        }, this);
        return found;
    },

    /** public: method[getLayersByServiceType]
     *  :param match: ``String`` service type the layer must match
     *  :param layers: ``Array`` of :class:``OpenLayers.Layer`` objects.  If not
     *                           defined, then use all map layers.
     *  :return:  ``Array`` of :class:``OpenLayers.Layer`` objects.
     *
     *  Returns layers that matching the specified service type.
     */
    getLayersByServiceType: function(match, layers) {
        layers = layers || this.mapPanel.map.layers;
        var found = [], type;
        Ext.each(layers, function(l) {
            type = (l.servicetype) ? l.servicetype : l.serviceType;
            type && type == match && found.push(l);
        }, this);
        return found;
    },

    /** public: method[setMapPanel]
     *  :param mapPanel: :class:``GeoExt.MapPanel``
     *  
     *  Sets the mapPanel property with providen MapPanel object OR guess one.
     */
    setMapPanel: function(mapPanel) {
        this.mapPanel = (mapPanel instanceof GeoExt.MapPanel)
            ? mapPanel : GeoExt.MapPanel.guess();
    },

    /** public: method[zoomToFeatureExtent]
     *  :param feature: :class:``OpenLayers.Feature.Vector``
     *  :param maxScale: ``Float`` Max scale allowed to zoom to.
     *  
     *  Zoom to the feature extent OR feature center with limited zoom scale.
     */
    zoomToFeatureExtent: function(feature, maxScale) {
        if (!this.mapPanel) {
            this.setMapPanel();
        }
        var map, bounds, zoom, maxZoom, u;
        if (!feature || !feature.geometry) {
            return;
        }
        map = this.mapPanel.map;
        if (maxScale) {
            bounds = feature.geometry.getBounds();
            zoom = map.getZoomForExtent(bounds);
            maxZoom = map.getZoomForResolution(
                OpenLayers.Util.getResolutionFromScale(maxScale, map.units));
            map.setCenter(
                bounds.getCenterLonLat(),
                (zoom > maxZoom) ? maxZoom : zoom
            );
        } else {
            map.zoomToExtent(feature.geometry.getBounds());
        }
    }
});
