/**
 * Copyright (c) 2008-2011 The Open Source Geospatial Foundation
 *
 * Published under the BSD license.
 * See http://svn.geoext.org/core/trunk/geoext/license.txt for the full text
 * of the license.
 */

Ext.namespace("GeoExt.ux");

/*
 * @requires Ext.ux.grid.Search.js
 */

if (Ext.ux.grid && Ext.ux.grid.Search) {

   GeoExt.ux.WFSSearch = Ext.extend(Ext.ux.grid.Search, {

        /**
         * APIProperty: querySeparator (string) The separator string used to 
         *     initiate multiple queries in the featureConfigs array
         */
        querySeparator: ',',
        
        /**
         * APIProperty: filter
         * {OGCFilter} The filter created by selected options
         */
        filter : null,
        
        /**
         * APIProperty: wfsFilterBuilderButton
         * linked parent GeoExt.ux.WFSFilterBuilderButton
         */
        wfsFilterBuilderButton : null,

        /**
         * APIProperty: width
         * {Integer} Default value of the 'width' property for this widget.
         */
        width: 200,

        /**
         * APIProperty: triggerDelay
         * {Integer} Delay in ms after keyup in searchfield to trigger a new
         *           new query.
         */
        triggerDelay: 1000,
		
        /** private: method[init]
         */
        init: function(grid) {
            GeoExt.ux.WFSSearch.superclass.init.call(this, grid);
            grid.on("afterrender", function(grid) {
                // when the menu hides, set focus on the search textfield
                this.menu.on("hide", function(menu) {
                    this.focus('', 10);
                }, this.field);
            }, this);
        },

        /** api: method[setFilter]
         *  :arg filter: ``OpenLayers.Filter`` Filter to be set on the WFS
         *                                     protocol.
         */
        setFilter: function(filter) {
            if (filter) {
                this.filter = filter;
            } else {
                this.filter = null;
            }
        },

        /** api: method[getFilter]
         *  :returns: ``OpenLayers.Filter``
         */
        getFilter: function() {
            return this.filter;
        },

        /** Method: clearSearch
         *  This method clear the search textfield.
         */
        clearSearch: function() {
            this.field.setValue('');
        },

        /**
         * Search Trigger click handler (executes the search, local or remote)
         * @private
         */
        onTriggerSearch:function() {
            // don't try to do request if field is readOnly
            if (this.field.readOnly) {
                return;
            }
	
            if(!this.field.isValid()) {
                return;
            }
            var val = this.field.getValue();
            var store = this.grid.store;

            // clear start (necessary if we have paging)
            if(store.lastOptions && store.lastOptions.params) {
                store.lastOptions.params[store.paramNames.start] = 0;
            }

            // get fields to search array
            var fields = [];
            this.menu.items.each(function(item) {
                if(item.checked) {
                    fields.push(item.dataIndex);
                }
            });

            var subFilters = [];
            Ext.each(val.split(this.querySeparator), function(query, index, queries) {
                if(query == '') return;
                
                var fieldFilters = [];
                Ext.each(fields, function(field, index, fields) {
                    if(field)
                    {
                        fieldFilters.push(new OpenLayers.Filter.Comparison({
                            type: OpenLayers.Filter.Comparison.LIKE,
                            property: field,
                            value: query,
                            matchCase: false
                        }));
                    }
                }, this);

                if(fieldFilters.length == 1) {
                    subFilters.push(fieldFilters[0]);
                } else if (fieldFilters.length > 1) {
                    subFilters.push(new OpenLayers.Filter.Logical({
                        type: OpenLayers.Filter.Logical.OR,
                        filters: fieldFilters
                    }));
                }
            }, this);

            var filter;
            if(subFilters.length == 1) {
                filter = subFilters[0];
            } else if (subFilters.length > 1) {
                filter = new OpenLayers.Filter.Logical({
                    type: OpenLayers.Filter.Logical.AND,
                    filters: subFilters
                });
            }

            // set this filter
            this.setFilter(filter);
			
            // disable field until end of store loading
            this.field.setReadOnly(true);
            var searchFieldEnable = function(store, records) {
                this.field.setReadOnly(false);
                this.field.focus();
                store.un("load", searchFieldEnable);
            };
            store.on("load", searchFieldEnable, this);

            // load the store
            store.load();
        },

        /** private: method[reconfigure]
         */
        reconfigure: function() {
            GeoExt.ux.WFSSearch.superclass.reconfigure.call(this);
            // if hideSearchFieldMenuOnCheck is set, hide the search menu
            // after an item has been checked.
            if (this.wfsFilterBuilderButton.hideSearchFieldMenuOnCheck) {
                this.menu.items.each(function(item, index, length) {
                    item.on("checkchange", function(checkItem, checked) {
                        checked && this.menu.hide();
                    }, this);
                }, this);
            }
			
        },
        
        /** private: method[onKeyUp]
         *  Complete override of the "onKeyUp" method of the parent class
         *  (is a copy with local modifications). Adds more 
         *  
         *  See parent method for more documentation.
         */
        onKeyUp:function(e, t, o) {
            // ignore special keys 
            if((e.isNavKeyPress() || e.isSpecialKey()) &&
               e.keyCode != e.BACKSPACE && e.keyCode != e.DELETE) {
                return;
            }

            var length = this.field.getValue().toString().length;
            if(0 === length || this.minChars <= length) {
                this.onTriggerSearch();
            }
        },
        
        /** private: method[onRender]
         *  Complete override of the "onRender" method of the parent class
         *  (is a copy with local modifications).  The original method has the 
         *  "buffer" value set to 300 in it but we want to be able to set it
         *  as the "triggerDelay" APIProperty.
         */
        onRender:function() {
            var panel = this.toolbarContainer || this.grid;
            var tb = 'bottom' === this.position ? panel.bottomToolbar : panel.topToolbar;
            // add menu
            this.menu = new Ext.menu.Menu();

            // handle position
            if('right' === this.align) {
                tb.addFill();
            }
            else {
                if(0 < tb.items.getCount()) {
                    tb.addSeparator();
                }
            }

            // add menu button
            tb.add({
                 text:this.searchText
                ,menu:this.menu
                ,iconCls:this.iconCls
            });

            // add input field (TwinTriggerField in fact)
            this.field = new Ext.form.TwinTriggerField({
                 width:this.width
                ,selectOnFocus:undefined === this.selectOnFocus ? true : this.selectOnFocus
                ,trigger1Class:'x-form-clear-trigger'
                ,trigger2Class:this.minChars ? 'x-hide-display' : 'x-form-search-trigger'
                ,onTrigger1Click:this.onTriggerClear.createDelegate(this)
                ,onTrigger2Click:this.minChars ? Ext.emptyFn : this.onTriggerSearch.createDelegate(this)
                ,minLength:this.minLength
            });

            // install event handlers on input field
            this.field.on('render', function() {
                // register quick tip on the way to search
                if((undefined === this.minChars || 1 < this.minChars) && this.minCharsTipText) {
                    Ext.QuickTips.register({
                         target:this.field.el
                        ,text:this.minChars ? String.format(this.minCharsTipText, this.minChars) : this.searchTipText
                    });
                }

                if(this.minChars) {
                    this.field.el.on({scope:this, buffer:this.triggerDelay, keyup:this.onKeyUp});
                }

                // install key map
                var map = new Ext.KeyMap(this.field.el, [{
                     key:Ext.EventObject.ENTER
                    ,scope:this
                    ,fn:this.onTriggerSearch
                },{
                     key:Ext.EventObject.ESC
                    ,scope:this
                    ,fn:this.onTriggerClear
                }]);
                map.stopEvent = true;
            }, this, {single:true});

            tb.add(this.field);

            // re-layout the panel if the toolbar is outside
            if(panel !== this.grid) {
                this.toolbarContainer.doLayout();
            }

            // reconfigure
            this.reconfigure();

            // keyMap
            if(this.shortcutKey && this.shortcutModifier) {
                var shortcutEl = this.grid.getEl();
                var shortcutCfg = [{
                     key:this.shortcutKey
                    ,scope:this
                    ,stopEvent:true
                    ,fn:function() {
                        this.field.focus();
                    }
                }];
                shortcutCfg[0][this.shortcutModifier] = true;
                this.keymap = new Ext.KeyMap(shortcutEl, shortcutCfg);
            }

            if(true === this.autoFocus) {
                this.grid.store.on({scope:this, load:function(){this.field.focus();}});
            }
			
        }
    });
}
