<?php
/**
* Classe de base des datastore, Lier aux fichier de configuration
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Classe de base des datastore, Lier aux fichier de configuration 
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Datastore 
* @author     Pascal Martin 
*/ 
class org_geoprisma_datastore_Datastore extends org_geoprisma_ConfigElement
{
    private $m_strName = null;
    private $m_objArrayLayers = null;
    private $m_objArrayOption = null;
    
    /**
    * @var org_geoprisma_service_Service
    */
    private $m_objService = null;

    /**
    * Construit un nouvelle instance de org_geoprisma_datastore_Datastore
    * 
    * @param string                        $pstrName         Datastore name
    * @param org_geoprisma_service_Service &$pobjService     Service object
    * @param string                        $pstrLayers       String of layers
    * @param array                         $pobjArrayOptions Datastore options
    * 
    * @return org_geoprisma_datastore_Datastore
    */
    public function __construct(
        $pstrName,
        org_geoprisma_service_Service &$pobjService,
        $pstrLayers=null,
        $pobjArrayOptions=null
    ) {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName);

        // name
        $this->m_strName = $pstrName;

        // service
        $this->m_objService = $pobjService;

        // layers
        if ($pstrLayers != null && $pstrLayers != "") 
        {
            $this->m_objArrayLayers = $this->getLayersFromString($pstrLayers);
        }

        // options
        $this->m_objArrayOption = new ArrayObject();
        if (is_array($pobjArrayOptions)) 
        {
            foreach ($pobjArrayOptions as $strOptionKey => $strOptionValue) 
            {
                $this->m_objArrayOption->append(
                    new org_geoprisma_option_Option(
                        $strOptionKey,
                        $strOptionValue
                    )
                );     
            }
        }
    }

    /**
    * Extract layer names from a string
    *
    * @param string $pstrLayers Layers separated by ','
    * 
    * @return ArrayObject of string
    */
    private function getLayersFromString($pstrLayers)
    {
        $objArrayLayers = new ArrayObject();    
 
        $objArrayLayersTemp = explode(",", $pstrLayers);
        foreach ($objArrayLayersTemp as $strLayer)
        {
            $strLayer = trim($strLayer);
            if ( $strLayer != "") 
            {
                $objArrayLayers->append($strLayer);
            }
        }

        return $objArrayLayers;
    }
    
    /**
    * Check if the passed layer are authorized
    * 
    * @param string $pstrLayer Layer Name
    * 
    * @return boolean true or false
    */
    public function isAuthorizedLayer($pstrLayer)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrLayer);
        if (array_search($pstrLayer, (array)$this->m_objArrayLayers) === false) 
        {
            return false;          
        }                
        return true;      
    }

    /**
    * Return the DataStore name
    *
    * @return string
    */
    public function getName()
    {
        return $this->m_strName;   
    }
    
    /**
    * Return the Service object linked to this DataStore
    * 
    * @return org_geoprisma_service_Service 
    */
    public function getService()
    {
        return $this->m_objService;   
    }

    /**
    * Return the layers array object
    * 
    * @return ArrayObject
    */
    public function getLayers()
    {
        return $this->m_objArrayLayers;
    }

    /**
    * Get layer string (separated by ',')
    *
    * @return string
    */
    public function getLayerString()
    {
        return implode(",", (array)$this->getLayers());
    }

    /**
    * Get all datastore options
    *
    * @return ArrayObject
    */
    public function getOptions()
    {
        return $this->m_objArrayOption;   
    }
    
    /**
    * Get a specific option using its name
    * 
    * @param string  $pstrName    Option name
    * @param boolean $pbSkipError Whether the method should skip throwing an
    *                             error if no option is found.
    * 
    * @return org_geoprisma_option_Option
    */
    public function getOption($pstrName, $pbSkipError=false)
    {
        if ($this->getOptions() != null) 
        {
            foreach ($this->getOptions() as $objOption)   
            {
                if ($objOption->getName() == $pstrName) 
                {
                    return $objOption;
                }    
            }
        }

        if ($pbSkipError) 
        {
            return false;
        }
        else
        {
            throw new org_geoprisma_exception_OptionNotFoundException($pstrName);
        }
    }
    
    /**
    *  Ajout le datastore courant (this) dans le xml      
    * 
    * @param DOMElement  &$pobjDomElementDatastores Noeud xml ou le datastore doit Ãªtre ajouter
    * @param DOMDocument &$pobjDomDocument          Document XMl ou le noeud xml datastore est ajouter, utiliser pour construire les noeuds
    * 
    * @return void
    */
    public function insertIntoXML(DOMElement &$pobjDomElementDatastores, DOMDocument &$pobjDomDocument)
    {
        $objDomElementDatastore = $pobjDomDocument->createElement('datastore'); 
        $pobjDomElementDatastores->appendChild($objDomElementDatastore);        
        $objDomElementName = $pobjDomDocument->createElement('name', $this->getName()); 
        $objDomElementService = $pobjDomDocument->createElement('service', $this->getService()->getName());
        $objDomElementDatastore->appendChild($objDomElementName);
        $objDomElementDatastore->appendChild($objDomElementService);       
        
        // params with options
        $objDomElementParams = $this->getDomElements(
            'params', $this->getOptions(), $pobjDomDocument
        );

        // params/layers adding
        if ($this->getLayers() != null) 
        {
            $objDomElementLayers = $pobjDomDocument->createElement(
                'layers', $this->getLayerString()
            ); 
            $objDomElementParams->appendChild($objDomElementLayers);
        }

        $objDomElementDatastore->appendChild($objDomElementParams);
    }
    
}

?>
