<?php
/**
* Class to read the disc and build a JSON array from the files.
* This is based on documentation found at:
* http://aariadne.com/filetree/
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Julien-Samuel Lacroix 
* @copyright 2010, Mapgears inc
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Class to read the disc and build a JSON array from the files.
* This is based on documentation found at:
* http://aariadne.com/filetree/
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Proxy
* @author     Julien-Samuel Lacroix
*/   
class org_geoprisma_proxy_file_FileTree
{
    var $m_strRootPath = '';

    /**
     * Build a FileTree 
     * 
     * @param string $pstrRootPath based path of the listing
     *
     * @return void
     */
    public function __construct($pstrRootPath="")
    {
        if ($pstrRootPath && is_dir($pstrRootPath)) 
        {
            $this->setRootPath($pstrRootPath);
        }
    }

    /**
     * Read the content of a file path and returns a JSON object of it
     * 
     * @param string $pstrFilePath       Directory to read
     * @param array  $pobjArrayFileList  Array of files
     * @param array  $pobjArrayResources Array of resources
     *
     * @return string JSON
     */
    public function get(
        $pstrFilePath,
        $pobjArrayFileList=array(),
        $pobjArrayResources=array()
    ) {
        $strFilePath = $this->m_strRootPath.'/'.$pstrFilePath;

        if (!$this->m_strRootPath || !is_dir($this->m_strRootPath) || is_file($strFilePath) || !is_dir($strFilePath)) 
        {
            return '[]';
        }

        if (count($pobjArrayFileList) <= 0) 
        {
            $pobjArrayFileList = $this->getFileList($strFilePath);
        }

        $objArrayDirectoryToJSON = array();
        foreach ($pobjArrayFileList as $strEntry)
        {
            if (substr($strEntry, 0, 1) != '.') 
            {
                $objArrayDirectoryToJSON[] = $this->getFileJSON($strFilePath, $strEntry, $pobjArrayResources);
            }
        }

        return '['.implode(',', $objArrayDirectoryToJSON).']';
    }

    /**
     * Read the content of a file path and returns a JSON object of it
     * 
     * @param string $pstrFilePath directory to read
     *
     * @return string JSON string
     */
    public function download($pstrFilePath)
    {
        $strFilePath = $this->m_strRootPath.'/'.$pstrFilePath;

        if (!$this->m_strRootPath || !is_dir($this->m_strRootPath) || !is_file($strFilePath)) 
        {
            return;
        }

        header("Content-type: application/force-download");
        header("Content-Transfer-Encoding: Binary");
        header("Content-length: ".filesize($strFilePath));
        header("Content-disposition: attachment; filename=\"".basename($strFilePath)."\"");
        readfile($strFilePath);

        return;
    }

    /**
    * Get file JSON.
    * 
    * @param string $pstrPath           Path of the file
    * @param string $pstrFile           Name of the file
    * @param array  $pobjArrayResources Array of resources
    * 
    * @return string JSON
    */
    public function getFileJSON($pstrPath, $pstrFile, $pobjArrayResources)
    {
        if (substr($pstrPath, -1) != '/') 
        {
            $pstrPath .= '/';
        }

        $objFileDescription = array(
            'text'     => $pstrFile,
            'disabled' => false,
            'leaf'     => (is_dir($pstrPath.$pstrFile)?false:true),
            'qtip'     => (is_file($pstrPath.$pstrFile)?'Size: '.filesize($pstrPath.$pstrFile).' bytes':'')
        );

        if (is_dir($pstrPath.$pstrFile)) 
        {
            $objFileDescription['cls'] = 'folder';
        }
        else if (strrpos($pstrFile, '.')) 
        {
            $objFileDescription['iconCls'] = 'file-'.strtolower(substr($pstrFile, strrpos($pstrFile, '.')+1));
        }

        if (isset($pobjArrayResources[$pstrFile])) 
        {
            $objFileDescription['osmresource'] = $pobjArrayResources[$pstrFile];
        }

        return json_encode($objFileDescription);
    }

    /**
    * Get file list.
    * 
    * @param string $pstrFilePath Path of the file
    * 
    * @return array
    */
    public function getFileList($pstrFilePath)
    {
        $strFilePath = $this->m_strRootPath.'/'.$pstrFilePath;

        $objArrayFileList = array();

        if (!is_dir($strFilePath)) 
        {
            return $objArrayFileList;
        }

        $objDirectory = dir($strFilePath);
        while (false !== ($strEntry = $objDirectory->read())) 
        {
            if (substr($strEntry, 0, 1) != '.') 
            {
                $objArrayFileList[] = $strEntry;
            }
        }
        $objDirectory->close();

        return $objArrayFileList;
    }

    /**
    * Get the root path.
    * 
    * @return string
    */
    private function getRootPath()
    {
        return $this->m_strRootPath;
    }

    /**
    * Set the root path.
    *
    * @param string $pstrRootPath The root path to set.
    *
    * @return void
    */
    private function setRootPath($pstrRootPath)
    {
        if (substr($pstrRootPath, -1) == '/') 
        {
            $pstrRootPath = substr($pstrRootPath, 0, -1);
        }

        $this->m_strRootPath = $pstrRootPath;

        return;
    }

    // #### UPLOAD ####

    /**
    * Returns current uploaded item
    *
    * @return array
    */
    public function getUploadedItem()
    {
        return $_FILES['x-filename'];
    }

    /**
    * Upload a file using current uploaded item.
    *
    * @param string $pstrBasePath The bae path of the file to upload.
    *
    * @return string
    */
    public function upload($pstrBasePath)
    {
        $objResponse = false;

        $strBasePath = realpath($this->getRootPath().'/'.$pstrBasePath);

        if (!$this->m_strRootPath || !is_dir($this->m_strRootPath) || !is_dir($strBasePath)) 
        {
            return;
        }

        $objArrayUploadedItem = $this->getUploadedItem();
        $strFilePath = $strBasePath.'/'.$objArrayUploadedItem['name'];

        // do not upload if file already exists
        if (is_file($strFilePath))
        {
            $objResponse = array(
                'success' => false,
                'errors' => array("message" => "File already exists")
            );
        }

        // UPLOAD_ERR_? checkup
        if ($objResponse === false
            && $objArrayUploadedItem['error'] !== UPLOAD_ERR_OK
        ) {
            $objResponse = array(
                'success' => false,
                'errors' => array(
                    "message" => $this->getFileUploadErrorMessage(
                        $objArrayUploadedItem['error']
                    )
                )
            );
        }

        // move_uploaded_file
        if ($objResponse === false)
        {
            $bMoved = move_uploaded_file(
                $objArrayUploadedItem['tmp_name'],
                $strFilePath
            );

            if ($bMoved)
            {
                $objResponse = array('success' => true);
            }
            else
            {
                $objResponse = array(
                    'success' => false,
                    'errors' => array("message" => "Could not upload file")
                );
            }
        }

        return json_encode($objResponse);
    }

    /**
    * Create a new directory
    *
    * @param string $pstrPath The path of the directory to create
    *
    * @return string
    */
    public function createDirectory($pstrPath)
    {
        $strPath = $this->getRootPath().'/'.$pstrPath;

        if (mkdir($strPath))
        {
            $objResponse = array('success' => true);
        }
        else
        {
            $objResponse = array(
                'success' => false,
                'errors' => array("error" => "Could not create directory")
            );
        }

        return json_encode($objResponse);
    }

    public function getFileUploadErrorMessage($piErrorCode)
    {
        $strMessage = '';

        switch ($piErrorCode) {
            case UPLOAD_ERR_INI_SIZE:
                $strMessage = 'The uploaded file exceeds the upload_max_filesize directive in php.ini';
                break;
            case UPLOAD_ERR_FORM_SIZE:
                $strMessage = 'The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form';
                break;
            case UPLOAD_ERR_PARTIAL:
                $strMessage = 'The uploaded file was only partially uploaded';
                break;
            case UPLOAD_ERR_NO_FILE:
                $strMessage = 'No file was uploaded';
                break;
            case UPLOAD_ERR_NO_TMP_DIR:
                $strMessage = 'Missing a temporary folder';
                break;
            case UPLOAD_ERR_CANT_WRITE:
                $strMessage = 'Failed to write file to disk';
                break;
            case UPLOAD_ERR_EXTENSION:
                $strMessage = 'File upload stopped by extension';
                break;
            default:
                $strMessage = 'Unknown upload error';
        }

        return $strMessage;
    }
}

?>
