<?php
/**
* Classe de base des widget, Lier aux fichier de configuration
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Classe de base des widget, Lier aux fichier de configuration 
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Widget 
* @author     Pascal Martin 
*/ 
abstract class org_geoprisma_widget_Widget extends org_geoprisma_ConfigElement
{
    private $m_strName = null;
    protected $m_objArrayOptions = null;
    protected $m_objArrayResources = null;
     
    /**
    * Construit un instance de org_geoprisma_widget_Widget
    * 
    * @param string $pstrName Widget name
    * 
    * @return org_geoprisma_widget_Widget
    */
    public function __construct($pstrName) 
    {
        $this->m_strName = $pstrName;
        $this->m_objArrayResources = new ArrayObject();
    }

    /**
    * Clone current object member variable
    * 
    * @return void
    */
    function __clone()
    {
        if ($this->m_objArrayOptions != null) 
        {
            $this->m_objArrayOptions = clone $this->m_objArrayOptions;
        }
    }
    
    /**
    * Retourne la liste options du widget
    * 
    * @return ArrayObject
    */
    public function getOptions()
    {
        return $this->m_objArrayOptions;
    }
    
    /**
    * Retourne l'option demander
    * 
    * @param string $pstrName Option name
    * 
    * @return org_geoprisma_option_Option
    */
    public function getOption($pstrName)
    {
        if ($this->m_objArrayOptions != null) 
        {
            foreach ($this->m_objArrayOptions as $objOption)   
            {
                if ($objOption->getName() == $pstrName) 
                {
                    return $objOption;
                }    
            }
        }
        throw new org_geoprisma_exception_OptionNotFoundException($pstrName);
    }
    
    /**
    * Return the widget name
    *
    * @return string
    */
    public function getName()
    {
        return $this->m_strName;   
    }
    
    /**
    * Set the widget name
    *
    * @param string $pstrName name of widget 
    * 
    * @return void
    */
    public function setName($pstrName)
    {
        $this->m_strName = $pstrName;   
    }
    
    /**
    * Retourne l'action du widget
    * 
    * @return string
    */
    public function getAction()
    {
        $strType = $this->getType();     
        include_once self::getWidgetPath($strType);    
        return call_user_func(array($strType, 'getAction')); 
    }
    
    /**
    * Retourne la path du widget pour le include
    * 
    * @param string $pstrType type de widget
    * 
    * @return string
    */
    public static function getWidgetPath($pstrType)
    {
        $objArrayDir = explode('_', $pstrType);        
        $iCount = count($objArrayDir);
        if ($iCount > 1) 
        {
            $strDirPath = '';      
            for ($iIter=0; $iIter<$iCount; $iIter++) 
            {
                $strDirPath .= '/'.$objArrayDir[$iIter];
            }
            return org_geoprisma_SettingImpl::getWidgetDir()."$strDirPath/".$objArrayDir[$iCount-1].".php";
        }
        else
        {            
            return org_geoprisma_SettingImpl::getWidgetDir()."/$pstrType/$pstrType.php";
        }        
    }

    /**
    *  Ajout le widget courant (this) dans le xml      
    * 
    * @param DOMElement  &$pobjDomElementWidgets Noeud xml ou le widget doit être ajouter
    * @param DOMDocument &$pobjDomDocument       Document XMl ou le noeud xml datastore est ajouter, utiliser pour construire les noeuds
    * 
    * @return void
    */   
    abstract public function insertIntoXML(DOMElement &$pobjDomElementWidgets, DOMDocument &$pobjDomDocument);
    
    /**
    * Retourne le type du widget
    * 
    * @return string
    */
    abstract public function getType();
    
    /**
    * Returns the 'servicetype' of the widget.
    * 
    * @return string
    */
    public function getServiceType()
    {
        $strType = $this->getType();     
        include_once self::getWidgetPath($strType);    
        return call_user_func(array($strType, 'getServiceType')); 
    }

    /**
    * Return the list of processes available to the widget.
    * 
    * @return array
    */
    public function getProcessList()
    {
        $strType = $this->getType();
        include_once self::getWidgetPath($strType);    
        return call_user_func(array($strType, 'getProcessList')); 
    }

    /**
    * Wether the widget needs to interact with a layer or not.  Defaults to
    *     false.
    *
    * @return string
    */
    public function needLayer()
    {
        $strType = $this->getType();     
        include_once self::getWidgetPath($strType);    
        return call_user_func(array($strType, 'needLayer')); 
    }

    /**
    * Retourne une liste d'information sur les datastores utilisé
    * 
    * @param ArrayObject $pobjArrayRessource Liste de nom de ressource
    * 
    * @return ArrayObject of array('resourcename' => ?, 'datastorename' => ?,  'servicename' => ? ) 
    */
    protected static function getDatastoresFromResources(ArrayObject $pobjArrayRessource)
    {
        $objArrayDatastore = new ArrayObject();
        foreach ($pobjArrayRessource as $strResource)
        {  
            try
            {
                $objResource = org_geoprisma_config_ConfigFactory::getConfig()->getResource($strResource);
                foreach ($objResource->getDatastores() as $objDatastore) 
                {
                    $objArrayDatastoreNew = array(
                        'resourcename' => $objResource->getName(), 
                        'datastorename' => $objDatastore->getName(),
                        'servicename' => $objDatastore->getService()->getName()
                    );
                    if (!in_array($objArrayDatastoreNew, (array)$objArrayDatastore)) 
                    {
                        $objArrayDatastore->append($objArrayDatastoreNew);
                    }
                }
            }
            catch(org_geoprisma_exception_ResourceNotFoundException $objError)
            {
                continue;   
            }
        }
        return $objArrayDatastore;
    }
    
    /**
    * Add or replace widget option
    * 
    * @param org_geoprisma_option_Option $pobjOption Option
    * 
    * @return void
    */
    public function addOrReplaceOption(org_geoprisma_option_Option $pobjOption)
    {
        // Try Replace
        if ($this->m_objArrayOptions != null) 
        {
            $iLength = $this->m_objArrayOptions->count();
            for ($iIter=0;$iIter<$iLength;++$iIter)
            {
                if ( $this->m_objArrayOptions->offsetGet($iIter)->getName() == $pobjOption->getName()) 
                {
                    $this->m_objArrayOptions->offsetSet($iIter, $pobjOption); 
                    return;      
                }
            }
            
            // or Add 
            $this->m_objArrayOptions->append($pobjOption);   
        }
        else
        {
            throw new Exception('Widget of type '. $this->getType().' do not support widget overload option' );   
        }
    }

    /**
    * Return the list of mandatory resource options a resource must have in 
    *     order to be linked to this widget.
    * 
    * @return array
    */
    public function getMandatoryResourceOptionList()
    {
        $strType = $this->getType();
        include_once self::getWidgetPath($strType);
        return call_user_func(array($strType, 'getMandatoryResourceOptionList')); 
    }

    /**
    * Return the list of non-mandatory resource options a resource can have to
    *     be used by this widget.
    * 
    * @return array
    */
    public function getOptionalResourceOptionList()
    {
        $strType = $this->getType();     
        include_once self::getWidgetPath($strType);    
        return call_user_func(array($strType, 'getAllResourceOptionList')); 
    }

    /**
    * Return the list of mandatory resource option values a resource must have
    *     in order to be linked to this widget.
    * 
    * @return array
    */
    public function getMandatoryResourceOptionValueList()
    {
        $strType = $this->getType();
        include_once self::getWidgetPath($strType);
        return call_user_func(
            array($strType, 'getMandatoryResourceOptionValueList')
        ); 
    }

    /**
    * Return the complete list of resource options (mandatory and optional) a 
    *     resource can have to be used by this widget.
    * 
    * @return string
    */
    public function getAllResourceOptionList()
    {
        $strType = $this->getType();     
        include_once self::getWidgetPath($strType);    
        return call_user_func_array(
            array($strType, 'getAllResourceOptionList'),
            array($strType)
        ); 
    }

    /**
    * Return resources used by this widget.  Needs to be overwritten in order
    *     to actually do something.
    * 
    * @return array
    */
    public function getResources()
    {
        $objArrayResourceNames = new ArrayObject();

        foreach ($this->m_objArrayResources as $objResource) 
        {
            $objArrayResourceNames->append($objResource->getName());
        }
        return $objArrayResourceNames;  
    }
    
    /**
    * Return datastores used by this widget.  Needs to be overwritten in order
    *     to actually do something.
    * 
    * @return array
    */
    public function getDatastores()
    {
        return $this->getDatastoresFromResources($this->getResources());
    }

    /**
    * Add 'org_geoprisma_resource_Resource' object to resource array object.
    *
    * @param org_geoprisma_resource_Resource $pobjResource Resource
    * 
    * @return void
    */
    public function addResource($pobjResource)
    {
        $this->m_objArrayResources->append($pobjResource);
    }

    /**
    * Wheter the widget needs to be cloned when generating the 'config_secur'
    *     dom while using new configs that separate widgets from resources.
    *
    * @return boolean
    */
    public function mustClone()
    {
        $strType = $this->getType();
        include_once self::getWidgetPath($strType);    
        return call_user_func(array($strType, 'mustClone')); 
    }

    /**
    * Get widget options from resource.
    *
    * @param org_geoprisma_resource_Resource $pobjResource Resource object
    *
    * @return boolean
    */
    public function getOptionsFromResource($pobjResource)
    {
        $strType = $this->getType();
        include_once self::getWidgetPath($strType);    
        return call_user_func_array(
            array($strType, 'getOptionsFromResource'),
            array($pobjResource, $strType)
        );
    }

    /**
    * Link resource to widget.
    *
    * @param org_geoprisma_resource_Resource $pobjResource Resource object
    * @param org_geoprisma_config_Config     $pobjConfig   Config object
    *
    * @return void
    */
    public function linkResource($pobjResource, $pobjConfig=null)
    {
        $strType = $this->getType();
        include_once self::getWidgetPath($strType);    
        return call_user_func_array(
            array($strType, 'linkResource'),
            array($pobjResource, $this, $pobjConfig)
        );
    }

    /**
    * Check if given resource has all required by this widget.  If the widget
    *    doesn't have any mandatory options, this method returns 'null'.
    *
    * @param org_geoprisma_resource_Resource $pobjResource Resource object
    *
    * @return boolean Wether the resource has all options required by the widget
    *                 or not.
    */
    public function resourceHasRequiredWidgetOptions($pobjResource)
    {
        $objArrayResourceOptions = $this->getMandatoryResourceOptionList();

        if ($objArrayResourceOptions === null) 
        {
            return null;
        }

        $bHasRequiredOptions = true;

        foreach ($objArrayResourceOptions as $strResourceOption) 
        {
            $objArrayOptions = $pobjResource->getOptions();
            $objRequiredOption = $pobjResource->getOption(
                $strResourceOption, true
            );

            if (!$objRequiredOption 
                || $objRequiredOption->getValue() === 'false'
            ) 
            {
                // There could be more than one option where's searching for.
                // When that happens, they are stored in one option with the
                // same name + 's', value = array()  Having the option defined
                // is enough (no need to check the values)
                $objRequiredOption = $pobjResource->getOption(
                    $strResourceOption."s", true
                );

                if (!$objRequiredOption) 
                {
                    $bHasRequiredOptions = false;
                    break;
                }
            }
            else
            {
                $objArrayValues = $this->getMandatoryResourceOptionValueList();
                if (isset($objArrayValues[$strResourceOption]) 
                    && $objArrayValues[$strResourceOption] != $objRequiredOption->getValue()
                ) 
                {
                    $bHasRequiredOptions = false;
                    break;
                }
            } 
        }

        return $bHasRequiredOptions;
    }
}

?>
